package unileipzig.isys.neuronviz.streamer;

import unileipzig.isys.neuronviz.kernel.DataStreamer;

/**
 * Class StandardInputStreamer
 * 
 */
public class StandardInputStreamer extends DataStreamer
{	
	public StandardInputStreamer()
	{
		setInputStream(System.in);

		this.streamingMode = DataStreamer.PIPE_MODE;
		this.threadPause = 10;
	}

	public int getNotifyIntervall()
	{
		return notifyIntervall;
	}

	public void setNotifyIntervall(int notifyIntervall)
	{
		this.notifyIntervall = notifyIntervall;
	}
	
}
