/***************************************************************************
 *   Copyright (C) 2005 by Dominic Schneider   *
 *   dominic@isyspc8   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "qdatasource.h"

/** \brief Class for reading blocks/lines from a pipe (e.g. stdin)
  * \author Dominic Schneider
  */
class QPipeReader : public QDataSource
{
    Q_OBJECT

private:
    int delay;
    FILE* f;

public:
    QPipeReader(int delay = 0, FILE* f=stdin);
    virtual void run();
    void setDelay(int delay) {this->delay = delay;}
    int  getDelay()  {return delay;}
};
