/***************************************************************************
 *   Copyright (C) 2005 by Robot Group Leipzig                             *
 *    martius@informatik.uni-leipzig.de                                    *
 *    fhesse@informatik.uni-leipzig.de                                     *
 *    der@informatik.uni-leipzig.de                                        *
 *    frankguettler@gmx.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   $Log: oneactivemultipassivecontroller.h,v $
 *   Revision 1.2  2008/04/17 14:54:45  martius
 *   randomGen added, which is a random generator with long period and an
 *    internal state. Each Agent has an instance and passed it to the controller
 *    and the wiring. This is good for
 *   a) repeatability on agent basis,
 *   b) parallel execution as done in ode_robots
 *
 *   Revision 1.1  2007/03/22 08:15:30  robot3
 *   Special variant of an multicontroller, which has one main (active) controller
 *   and an arbitrary number of passive controllers. These passive controllers are like active controllers, but aren't be able to manipulate the motor values forwarded
 *   to the Agent. See ode_robots/simulations/MI_Simu for an example.
 *

 *                                                                         *
 ***************************************************************************/
#ifndef __ONEACTIVEMULTIPASSIVECONTROLLER_H
#define __ONEACTIVEMULTIPASSIVECONTROLLER_H

#include "abstractmulticontroller.h"

/**
 * class (interface) for using multiple controller, the first one is
 * the active one, which generates motor values. The other controllers
 * are passive and cannot affect the motor values.
 *
 */
class OneActiveMultiPassiveController : public AbstractMultiController {
public:

  /// contructor (hint: use $ID$ for revision)
	OneActiveMultiPassiveController(AbstractController* controller, std::string controllerName);

	virtual ~OneActiveMultiPassiveController();

/****************************************************************************/
/*	AbstractMultiController should implement the following classes:        	*/
/*	AbstractController, Configurable, Inspectable, Storeable	            */
/****************************************************************************/


/****************************************************************************/
/*	BEGIN methods of AbstractController                                 	*/
/****************************************************************************/

  /** initialisation of the controller with the given sensor/ motornumber
   * Must NORMALLY be called before use. For all multicontroller
   * call first AbstractMultiController::init(sensornumber,motornumber)
   * if you overwrite this method
   */
	virtual void init(int sensornumber, int motornumber, RandGen* randGen = 0);


	  /** performs one step (includes learning).
      Calculates motor commands from sensor inputs.
	@param sensors sensors inputs scaled to [-1,1]
	@param sensornumber length of the sensor array
	@param motors motors outputs. MUST have enough space for motor values!
	@param motornumber length of the provided motor array
  */
	virtual void step(const sensor* sensors, int sensornumber,
	                  motor* motors, int motornumber);

  /** performs one step without learning.
	@see step
  */
	virtual void stepNoLearning(const sensor* sensors , int sensornumber,
	                            motor* motors, int motornumber);


/****************************************************************************/
/*	END methods of AbstractController                                 	    */
/****************************************************************************/

/****************************************************************************/
/*	BEGIN methods of Configurable                                       	*/
/****************************************************************************/



/****************************************************************************/
/*	END methods of Configurable                                      	    */
/****************************************************************************/

/****************************************************************************/
/*	BEGIN methods of Inspectable                                          	*/
/****************************************************************************/

  /** The list of the names of all internal parameters given by getInternalParams().
      The naming convention is "v[i]" for vectors
       and "A[i][j]" for matrices, where i, j start at 0.
	@return: list of keys
   */
	virtual iparamkeylist getInternalParamNames() const {
		return AbstractMultiController::getInternalParamNames();
		// return all parameters from the other controllers
		// return params from multicontroller itself
	}

	  /** @return: list of values
   */
	virtual iparamvallist getInternalParams() const {
		return AbstractMultiController::getInternalParams();
		// return all parameters from the other controllers
		// return params from multicontroller itself
	}

/****************************************************************************/
/*	END methods of Inspectable                                       	    */
/****************************************************************************/

/****************************************************************************/
/*	BEGIN methods of Storeable                                           	*/
/****************************************************************************/

  /** stores the object to the given file stream (binary).
  */
	virtual bool store(FILE* f) const {
		return controller->store(f);
		// store the other controllers
		// store values from multicontroller itself
	}

  /** loads the object from the given file stream (binary).
  */
	virtual bool restore(FILE* f) {
		return controller->restore(f);
		// restore the other controllers
		// restore values from multicontroller itself
	}

/****************************************************************************/
/*	END methods of Storeable                                          	    */
/****************************************************************************/

protected:
	motor* passiveMotors;
};

#endif
